# -*- coding: utf-8 -*-
"""
Created on Thu Sep 10 11:58:46 2015

In this example an actually measured signal is deconvolved and the result compared to a reference measurement signal.

.. moduleauthor:: Sascha Eichstaedt (sascha.eichstaedt@ptb.de)
"""

from matplotlib.pyplot import *
from GUM2DFT import GUM_DFT,GUM_iDFT,GUMdeconv, AmpPhase2DFT, GUM_multiply

colors = [[0.1,0.6,0.5], [0.9,0.2,0.5], [0.9,0.5,0.1]]

# low-pass filter for deconvolution
def lowpass(f,fcut=80e6):
    return 1/(1+1j*f/fcut)**2


#%% reference data
ref_file = np.loadtxt("data/reference_signal.dat")
time = ref_file[:,0]
ref_data = ref_file[:,1]
Ts = 2e-9
N = len(time)

#%% measured hydrophone output signal
meas = np.loadtxt("data/measured_signal.dat")
y = meas[:,1]
# assumed noise std
noise_std = 4e-4
Uy = noise_std**2

#%% hydrophone calibration data
calib = np.loadtxt("data/calib_PA055.dat")
f = calib[:,0]
FR = calib[:,1]*np.exp(1j*calib[:,3])
Nf = 2*(len(f)-1)

uAmp = calib[:,2]
uPhas= calib[:,4]

#%% low pass filter for deconvolution regularization
HL = lowpass(f)
# inverse system for input estimation    
Hc = FR/np.abs(HL)
uAmp /= np.abs(HL)
H = np.r_[np.real(Hc), np.imag(Hc)]

UAP = np.r_[uAmp,uPhas*np.pi/180]**2
n = len(UAP)/2

#%% estimation of measurand using quasi-static approach
f_awf = 2.927*1e6
M_awf = 0.19341
uM_awf = M_awf*0.05
xh_awf = y / M_awf
uxh_awf= np.sqrt( Uy/M_awf**2 + y**2/M_awf**4*uM_awf**2 )
    
#%%
print( "Propagating uncertainty associated with measurement through DFT")
Y,UY = GUM_DFT(y,Uy,N=Nf)
# propagation to real/imag
print("Propagating uncertainty associated with calibration data to real and imag part")
UH = AmpPhase2DFT(np.abs(Hc),np.unwrap(np.angle(Hc))*np.pi/180,UAP)[1]
# propagation through deconvolution equation
print('Propagating uncertainty through the deconvolution')
XH,UXH = GUMdeconv(H,Y,UH,UY)
# propagation back to time domain
print("Propagating uncertainty associated with the estimate back to time domain\n")
xh,Uxh = GUM_iDFT(XH,UXH,Nx=N)

ux = np.sqrt(np.diag(Uxh))


uAmp = calib[:,2]
uPhas= calib[:,4]
UAP2 = np.r_[uAmp,uPhas*np.pi/180]**2
H2 = np.r_[np.real(FR), np.imag(FR)]
HL = np.r_[np.real(HL), np.imag(HL)]
# propagation to real/imag
print("Propagating uncertainty associated with calibration data to real and imag part")
UH2 = AmpPhase2DFT(np.abs(FR),np.unwrap(np.angle(FR))*np.pi/180,UAP2)[1]
# propagation through deconvolution equation
print('Propagating uncertainty through the deconvolution')
XH2,UXH2 = GUMdeconv(H2,Y,UH2,UY)
# application of low-pass filter
XH2, UXH2 = GUM_multiply(XH2,UXH2,HL)
# propagation back to time domain
print("Propagating uncertainty associated with the estimate back to time domain\n")
xh2,Uxh2 = GUM_iDFT(XH2,UXH2,Nx=N)
ux2 = np.sqrt(np.diag(Uxh))
    
# calculation of peak values
Nt = np.nonzero((time <= 2e-6))[0][-1]
u_ref = 0.2    
max_peaks = np.array([ref_data[:Nt].max(),xh[:Nt].max()])
min_peaks = np.array([ref_data[:Nt].min(),xh[:Nt].min()])
unc_max = np.array([u_ref,ux[xh[:Nt].argmax()]])
unc_min = np.array([u_ref,ux[xh[:Nt].argmin()]])
    
#%% plotting results
    
figure(1);clf()
plot(time*1e6,xh,label="estimate (GUM2DFT)",linewidth=2,color=colors[0])
plot(time*1e6,xh2,label="estimate (GUM2DFT 2)",linewidth=2,color=colors[-1])
plot(time*1e6,xh_awf,"-.",label="estimate (quasi-static)",linewidth=2,color=colors[2])
plot(time*1e6,ref_data,"--",label="reference data",linewidth=2,color=colors[1])
fill_between(time*1e6,xh+2*ux,xh-2*ux,alpha=0.2,color=colors[0])
fill_between(time*1e6,xh_awf+2*uxh_awf,xh_awf-2*uxh_awf,alpha=0.2,color=colors[2])
xlabel(r"time / $\mu$s",fontsize=22)
ylabel("signal amplitude / MPa",fontsize=22)
tick_params(which="major",labelsize=18)
legend(loc="upper left",fontsize=18,fancybox=True)
xlim(0,2)
    
dB = lambda a: 20*np.log10(np.abs(a))
figure(2);clf()
plot(f*1e-6,dB(FR),label="hydrophone",linewidth=2)
plot(f*1e-6,dB(1/Hc),label="deconvolution",linewidth=2)
plot(f*1e-6,dB(FR/Hc),"--",linewidth=0.5)
legend(loc="best",fontsize=18)
xlim(0.5,100); ylim(-40,40)
xlabel("frequency / MHz",fontsize=22)
ylabel("frequency response amplitude / dB",fontsize=22)
tick_params(which="both",labelsize=18)

figure(3);clf()
subplot(211)
errorbar(f*1e-6,np.abs(Hc),2*np.sqrt(UAP[:len(UAP)//2]),fmt=".-",alpha=0.2,color=colors[0])
xlim(0.5,80);ylim(0.04,0.24)
xlabel("frequency / MHz",fontsize=22); tick_params(which="both",labelsize=18)
ylabel("amplitude / V/MPa",fontsize=22)
subplot(212)
errorbar(f*1e-6,np.unwrap(np.angle(Hc))*np.pi/180,2*UAP[len(UAP)//2:],fmt=".-",alpha=0.2,color=colors[0])
xlim(0.5,80);ylim(-0.2,0.3)
xlabel("frequency / MHz",fontsize=22); tick_params(which="both",labelsize=18)
ylabel("phase / rad",fontsize=22)
    
figure(4);clf()
plot(time*1e6,ux,label="GUM2DFT",linewidth=2,color=colors[0])
plot(time*1e6,ux2,label="GUM2DFT 2",linewidth=2,color=colors[-1])
xlabel(r"time / $\mu$s",fontsize=22)
ylabel("uncertainty / MPa",fontsize=22)
tick_params(which="major",labelsize=18)
# xlim(0,2)

show()