# -*- coding: utf-8 -*-
"""

Comparison of the effect of different window functions. Note that the warning issued by GUM2DFT
function DFT2AmpPhase is ignored here, because only a comparison of the different windows is
intended, for which the achieved precision appears to be enough.

"""

from matplotlib.pyplot import *
from scipy.signal import windows

from GUM2DFT import GUM_DFT,DFT2AmpPhase,apply_window
from tools import tukeywin

Ts = 2e-9
time = np.arange(0,8e-6,Ts)
N = len(time)

#%% original signal
freqs = [2e6,5e6,10e6,12e6]
amps  = [0.5, 2.2, 2.5, 1.8]
y = np.array([ a*np.sin(2*np.pi*fr*time) for a,fr in zip(amps,freqs) ]).sum(axis=0)
Uy = 1e-2

#%% w/o window
rect = np.ones_like(time)
Y,UY = GUM_DFT(y,Uy,window=rect)
A,P,UAP = DFT2AmpPhase(Y,UY,keep_sparse = True)
fY = np.arange(0,N//2+1)/(N*Ts)

#%% with Tukey window
tukey = tukeywin(len(y),alpha=0.5)
yw,Uyw = apply_window(y,Uy,tukey)
Yw,UYw = GUM_DFT(y,Uy,window=tukey)
Aw,Pw,UAPw = DFT2AmpPhase(Yw,UYw,keep_sparse = True)

#%% with Hann window
hann = windows.hann(len(y))
yw2,Uyw2 = apply_window(y,Uy,hann)
Yw2,UYw2 = GUM_DFT(y,Uy,window=hann)
Aw2,Pw2,UAPw2 = DFT2AmpPhase(Yw2,UYw2,keep_sparse = True)

#%%
figure(1);clf()
plot(time*1e6,y,label="rectangular window")
plot(time*1e6,yw,label="Tukey window")
plot(time*1e6,yw2,label="Hann window")
xlabel(r"time / $\mu$s",fontsize=20)
ylabel("amplitude / au",fontsize=20)
legend(loc="best")

figure(2);clf()
plot(time*1e6,rect*np.sqrt(Uy),label="rectangular window")
plot(time*1e6,np.sqrt(Uyw),label="Tukey window")
plot(time*1e6,np.sqrt(Uyw2),label="Hann window")
legend(loc="best")
ylim(0,1.2*np.sqrt(Uy))
xlabel(r"time / $\mu$s",fontsize=20)
ylabel("window amplitude / au",fontsize=20)

uAmp  = np.sqrt(UAP.data[0][:N//2+1])/np.sum(rect)
uAmpw = np.sqrt(UAPw.data[0][:N//2+1])/np.sum(tukey)
uAmpw2= np.sqrt(UAPw2.data[0][:N//2+1])/np.sum(hann)

figure(3);clf()
errorbar(fY[:N//2+1]*1e-6,A/np.sum(rect),2*uAmp,fmt=".-",label="rectangular window")
errorbar(fY[:N//2+1]*1e-6,Aw/np.sum(tukey),2*uAmpw,fmt=".-",label="Tukey window")
errorbar(fY[:N//2+1]*1e-6,Aw2/np.sum(hann),2*uAmpw2,fmt=".-",label="Hann window")
xlim(0,15)
legend(loc="best")
for fr in freqs:
    axvline(fr*1e-6,linestyle="--",color="k")
xlabel(r"frequency / MHz",fontsize=20)
ylabel("DFT amplitude / au",fontsize=20)


print("\nDifference of endpoints w/o window \t",end=" ")
print("%g" %(np.abs(y[0]-y[-1])))

print("Difference of endpoints with Tukey window \t",end=" ")
print("%g" %(np.abs(yw[0]-yw[-1])))

show()