# -*- coding: utf-8 -*-


import numpy as np

def make_semiposdef(matrix,maxiter=10,tol=1e-12):

	n,m = matrix.shape
	if n!=m:
		raise ValueError("Matrix has to be quadratic")
	matrix = 0.5*(matrix+matrix.T)
	e = np.real(np.linalg.eigvals(matrix)).min()
	count = 0
	while e<tol and count<maxiter:
		matrix += (np.abs(e)+tol)*np.eye(n)
		e = np.real(np.linalg.eigvals(matrix)).min()
		count += 1
	return matrix

def multivariate_MonteCarlo(mean,covariance,modelfun,runs):
	"""
	Carry out multivariate Monte Carlo propagation
	Args:
	    mean: vector of best estimate
	    covariance: matrix of uncertainties either as covariance matrix or as vector of uncertainties (i.e. zero correlation)
	    modelfun: function which takes arrays of dimension of mean and returns model output
	    runs: number of Monte Carlo trials

	Returns:
	    vector of model outputs, mean and covariance
	"""
	try:
		test = modelfun(mean)
		output_dim = len(test)
	except:
		raise ValueError("Model function appears to be wrong.")

	if len(covariance.shape)==1: # zero correlation
		MCinputs = np.tile(mean,(runs,1)) + np.random.randn(runs,len(mean))*np.tile(covariance,(runs,1))
		model_outputs = np.zeros((runs,output_dim))
		for k in range(runs):
			model_outputs[k,:] = modelfun(MCinputs[k,:])
	else:
		model_outputs = np.random.multivariate_normal(mean,covariance,runs)

	estimate = model_outputs.mean(axis=0)
	uncertainty = np.cov(model_outputs,rowvar = False)
	return model_outputs, estimate, uncertainty



def tukeywin(window_length, alpha=0.5):
	'''The Tukey window, also known as the tapered cosine window, can be regarded as a cosine lobe of width \alpha * N / 2
	that is convolved with a rectangle window of width (1 - \alpha / 2). At \alpha = 1 it becomes rectangular, and
	at \alpha = 0 it becomes a Hann window.

	downloaded from  http://leohart.wordpress.com/2006/01/29/hello-world/ at 2014-03-13

	Reference
	J.W. Tukey: An introduction to the calculations of numerical spectrum analysis. In: Spectral Analysis of Time Series. 1967, S. 25–46
	---------

	'''

	# Special cases
	if alpha <= 0:
		return np.ones(window_length) #rectangular window
	elif alpha >= 1:
		return np.hanning(window_length)

	# Normal case
	x = np.linspace(0, 1, window_length)
	w = np.ones(x.shape)

	# first condition 0 <= x < alpha/2
	first_condition = x<alpha/2
	w[first_condition] = 0.5 * (1 + np.cos(2*np.pi/alpha * (x[first_condition] - alpha/2) ))

	# second condition already taken care of

	# third condition 1 - alpha / 2 <= x <= 1
	third_condition = x>=(1 - alpha/2)
	w[third_condition] = 0.5 * (1 + np.cos(2*np.pi/alpha * (x[third_condition] - 1 + alpha/2)))

	return w